//go:build go1.18
// +build go1.18

package generic

import (
	"errors"
	"fmt"
)

func NewContainer[T any](cType T) *Container[T] {
	return &Container[T]{
		elements: make(map[string]T),
	}
}

type Container[T any] struct {
	elements map[string]T
}

func (c *Container[T]) Add(element T) error {
	s, ok := any(element).(fmt.Stringer)
	if !ok {
		return errors.New("unknown element, does not implement fmt.Stringer{}")
	}

	if _, nok := c.elements[s.String()]; nok {
		return fmt.Errorf("element '%s' does already exists", s.String())
	}

	c.elements[s.String()] = element
	return nil
}

func (c *Container[T]) Get(name string) (T, error) {
	p, ok := c.elements[name]
	if !ok {
		var result T
		return result, fmt.Errorf("unknown element '%s' given", name)
	}

	return p, nil
}
